#!/usr/local/bin/perl

# strict
use strict;
use vars qw(%init $time);

# SQLite
use DBI;

# タイムゾーン設定
$ENV{'TZ'} = 'JST-9';

# flatseek-makedb.cgi (2009/08/16)

# 初期設定
%init = (

	# 検索データベースのパス
	db   => './flatseek-db.sqlite',

	# 検索ディレクトリのパス (最後に / を入れない)
	dir  => '../../public_html',

	# 検索対象設定 (ファイル名正規表現と解析ルーチンの組み合わせ)
	suf  => [

		{ match => qr/\.html?$/, parse => \&p_html },
		{ match => qr/\.txt$/,   parse => \&p_text },

	],

	# UTF-8以外のファイルを検索対称にする場合は変換ルーチン
#	conv => sub { use Jcode; Jcode::convert($_[0],'utf8'); },

); # init

#---

# main

{

	# 初期設定検査
	$time = time;

	# データベース更新
	&makedb();

}

exit;

#---

# subroutine

# データベース更新
sub makedb{
	# データベース,追加ルーチン
	my($dbh,$sth);

	# http-header
	$| = 1;
	print "Content-Type: text/plain; charset=UTF-8\n\n";

	# ファイル解析
	local $@;
	eval{

	$dbh = DBI->connect("DBI:SQLite:$init{'db'}.temp",'','',{ RaiseError => 1, PrintError => 1 });
	$dbh->do('CREATE TABLE flatseek (id INTEGER PRIMARY KEY, dir TEXT, name TEXT, url TEXT, type TEXT, pid TEXT, date TEXT, size TEXT, title TEXT, tag TEXT, tid TEXT, desc TEXT, text TEXT);');
	$dbh->do('BEGIN TRANSACTION;');

	$sth = $dbh->prepare('INSERT INTO flatseek (dir, name, url, type, pid, date, size, title, tag, tid, desc, text) VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?);');
	&p_list($sth,'',&itable());

	$dbh->do('END TRANSACTION;');
	$dbh->disconnect();
	rename("$init{'db'}.temp",$init{'db'});

	}; # db-eval

	# 更新結果表示
	print $@ if($@);
	print 'OK.';
	exit;
} # makedb end

# ファイル一覧取得
sub p_list{
	# データベース,ディレクトリ,索引テーブル
	my($sth,$dir,$ilist) = @_;
	# ファイル設定
	my($chk);

	# ディレクトリ解析
	local $_;
	local *D;
	opendir(D,"$init{'dir'}/$dir") || return();
	foreach(sort(grep { /^[^\.]/ } readdir(D))){
		if(-d "$init{'dir'}/$dir/$_"){
			&p_list($sth,"$dir/$_",$ilist);
		}elsif(-f _){
			foreach $chk (@{$init{'suf'}}){
				next if(!/$chk->{'match'}/);
				&p_item($sth,$dir,$_,$chk->{'parse'},$ilist);
				last;
			}
		}
	}
	closedir(D);
} # p_list end

# ファイル共通解析
sub p_item{
	# データベース,ディレクトリ,ファイル名,解析ルーチン,索引テーブル
	my($sth,$dir,$name,$parse,$ilist) = @_;
	# ファイル情報
	my($type,$url,$pid,$date,$size,$title,$tag,$tid,$desc,$text);

	# ファイル情報取得
	($type,$url,$pid,$date,$size,$title,$tag,$tid,$desc,$text) = $parse->($dir,$name);
	return() if(!$type);
	$url  = "$dir/$name"           if($url  eq '');
	$date = &mktime($dir,$name)    if($date eq '');
	$tid  = &mktidx($title,$ilist) if($tid  eq '');

	# データベース追加
	$sth->execute($dir,$name,$url,$type,$pid,$date,$size,$title,$tag,$tid,$desc,$text);
	print "$url OK.\n";
} # p_item

#---

# htmlファイル解析
sub p_html{
	# ディレクトリ,ファイル名
	my($dir,$name) = @_;
	# ファイル情報
	my($data,$size,$title,$desc);

	# ファイル解析
	&fread($dir,$name,\$data);
	$title = ($data =~ /<(title|h1)>\s*(\S[^\n]*?)\s*<\/\1>/i) ? &escape(1,$2) : &escape(0,$name);
	$size  = length($data);
	&escape(1,\$data);
	$desc  = &mkdesc($data,256);

	# return
	return('text','','','',$size,$title,'','',$desc,$data);
} # p_html end

# 通常ファイル解析
sub p_text{
	# ディレクトリ,ファイル名
	my($dir,$name) = @_;
	# ファイル情報
	my($data,$size,$title,$desc);

	# ファイル解析
	&fread($dir,$name,\$data);
	$title = &escape(0,$name);
	$size  = length($data);
	&escape(0,\$data);
	$desc  = &mkdesc($data,256);

	# return
	return('text','','','',$size,$title,'','',$desc,$data);
} # p_text end

#---

# ファイル時刻作成
sub mktime{
	my($dir,$name) = @_;
	my @temp = localtime($time - (24 * 60 * 60 * (-M "$init{'dir'}/$dir/$name")));
	return(sprintf('%04d%02d%02d%02d%02d%02d',($temp[5] + 1900),($temp[4] + 1),@temp[3,2,1,0]));
} # mktime end

# ファイル索引作成
sub mktidx{
	my($title,$ilist) = @_;
	while($title =~ /([a-zA-Z0-9]|[\xc0-\xf7][\x80-\xbf]{1,3})/g){
		return($ilist->{$1}) if(exists($ilist->{$1}));
	}
	return('漢字その他');
} # mktidx end

# ファイル概要作成
sub mkdesc{
	my($text,$mlen) = @_;
	return($text) if(length($text) <= $mlen);
	$text = substr($text,0,($mlen - 4));
	$text =~ s/\&[^\;]*$//;
	return($text . ' ...');
} # mkdesc end

#---

# ファイル読み込み
sub fread{
	my($dir,$name,$data) = @_;
	local *R;
	open(R,"< $init{'dir'}/$dir/$name") || return();
	read(R,$$data,-s "$init{'dir'}/$dir/$name");
	close(R);
	$init{'conv'}->($data) if(ref($init{'conv'}) eq 'CODE');
} # fread end

# 文字列エスケープ
sub escape{
	# 動作モード(0=通常変換,1=多重回避変換),変換する文字列
	my($mode,$text) = @_;

	# 変換処理
	my $tmp = (ref($text) eq 'SCALAR') ? $text : \$text;
	if($mode == 1){
		$$tmp =~ s/<\!--.*?-->//sg;
		$$tmp =~ s/<[^>]*?>//g;
		$$tmp =~ s/&(?!(?:amp|lt|gt|quot|nbsp|\#\d+);)/&amp;/ig;
	}else{
		$$tmp =~ s/&/&amp;/g;
	}
	$$tmp =~ s/</&lt;/g;
	$$tmp =~ s/>/&gt;/g;
	$$tmp =~ s/"/&quot;/g; #"
	$$tmp =~ s/'/&#39;/g;  #'
	$$tmp =~ tr/\x00-\x09\x0b-\x0c\x0e-\x1f//d;
	$$tmp =~ s/\x0d\x0a/\n/g;
	$$tmp =~ tr/\x0d\x0a/\n\n/;
	$$tmp =~ s/^\s*//mg;
	$$tmp =~ s/\s*$//mg;
	$$tmp =~ s/\n{2,}/\n/g;
	return((ref($text) eq 'SCALAR') ? '' : $text);
} # escape end

# 索引テーブル作成
sub itable{
	# テーブル情報
	my(%ilist,$list,$char);
	my @table = (
		[ 'あ' , 'ア' , 'ｱ' , 'ぁ' , 'ァ' , 'ｧ' ],
		[ 'い' , 'イ' , 'ｲ' , 'ぃ' , 'ィ' , 'ｨ' ],
		[ 'う' , 'ウ' , 'ｳ' , 'ぅ' , 'ゥ' , 'ｩ' ],
		[ 'え' , 'エ' , 'ｴ' , 'ぇ' , 'ェ' , 'ｪ' ],
		[ 'お' , 'オ' , 'ｵ' , 'ぉ' , 'ォ' , 'ｫ' ],
		[ 'か' , 'カ' , 'ｶ' , 'が' , 'ガ' ],
		[ 'き' , 'キ' , 'ｷ' , 'ぎ' , 'ギ' ],
		[ 'く' , 'ク' , 'ｸ' , 'ぐ' , 'グ' ],
		[ 'け' , 'ケ' , 'ｹ' , 'げ' , 'ゲ' ],
		[ 'こ' , 'コ' , 'ｺ' , 'ご' , 'ゴ' ],
		[ 'さ' , 'サ' , 'ｻ' , 'ざ' , 'ザ' ],
		[ 'し' , 'シ' , 'ｼ' , 'じ' , 'ジ' ],
		[ 'す' , 'ス' , 'ｽ' , 'ず' , 'ズ' ],
		[ 'せ' , 'セ' , 'ｾ' , 'ぜ' , 'ゼ' ],
		[ 'そ' , 'ソ' , 'ｿ' , 'ぞ' , 'ゾ' ],
		[ 'た' , 'タ' , 'ﾀ' , 'だ' , 'ダ' ],
		[ 'ち' , 'チ' , 'ﾁ' , 'ぢ' , 'ヂ' ],
		[ 'つ' , 'ツ' , 'ﾂ' , 'づ' , 'ヅ' , 'っ' , 'ッ' , 'ｯ' ],
		[ 'て' , 'テ' , 'ﾃ' , 'で' , 'デ' ],
		[ 'と' , 'ト' , 'ﾄ' , 'ど' , 'ド' ],
		[ 'な' , 'ナ' , 'ﾅ' ],
		[ 'に' , 'ニ' , 'ﾆ' ],
		[ 'ぬ' , 'ヌ' , 'ﾇ' ],
		[ 'ね' , 'ネ' , 'ﾈ' ],
		[ 'の' , 'ノ' , 'ﾉ' ],
		[ 'は' , 'ハ' , 'ﾊ' , 'ば' , 'バ' , 'ぱ' , 'パ' ],
		[ 'ひ' , 'ヒ' , 'ﾋ' , 'び' , 'ビ' , 'ぴ' , 'ピ' ],
		[ 'ふ' , 'フ' , 'ﾌ' , 'ぶ' , 'ブ' , 'ぴ' , 'プ' ],
		[ 'へ' , 'ヘ' , 'ﾍ' , 'べ' , 'ベ' , 'ぺ' , 'ペ' ],
		[ 'ほ' , 'ホ' , 'ﾎ' , 'ぼ' , 'ボ' , 'ぽ' , 'ポ' ],
		[ 'ま' , 'マ' , 'ﾏ' ],
		[ 'み' , 'ミ' , 'ﾐ' ],
		[ 'む' , 'ム' , 'ﾑ' ],
		[ 'め' , 'メ' , 'ﾒ' ],
		[ 'も' , 'モ' , 'ﾓ' ],
		[ 'や' , 'ヤ' , 'ﾔ' , 'ゃ' , 'ャ' , 'ｬ' ],
		[ 'ゆ' , 'ユ' , 'ﾕ' , 'ゅ' , 'ュ' , 'ｭ' ],
		[ 'よ' , 'ヨ' , 'ﾖ' , 'ょ' , 'ョ' , 'ｮ' ],
		[ 'ら' , 'ラ' , 'ﾗ' ],
		[ 'り' , 'リ' , 'ﾘ' ],
		[ 'る' , 'ル' , 'ﾙ' ],
		[ 'れ' , 'レ' , 'ﾚ' ],
		[ 'ろ' , 'ロ' , 'ﾛ' ],
		[ 'わ' , 'ワ' , 'ﾜ' , 'ゎ' , 'ヮ' ],
		[ 'を' , 'ヲ' , 'ｦ' ],
		[ 'ん' , 'ン' , 'ﾝ' ],
		[  '0' , '０' ],
		[  '1' , '１' ],
		[  '2' , '２' ],
		[  '3' , '３' ],
		[  '4' , '４' ],
		[  '5' , '５' ],
		[  '6' , '６' ],
		[  '7' , '７' ],
		[  '8' , '８' ],
		[  '9' , '９' ],
		[  'A' , 'Ａ' , 'a' , 'ａ' ],
		[  'B' , 'Ｂ' , 'b' , 'ｂ' ],
		[  'C' , 'Ｃ' , 'c' , 'ｃ' ],
		[  'D' , 'Ｄ' , 'd' , 'ｄ' ],
		[  'E' , 'Ｅ' , 'e' , 'ｅ' ],
		[  'F' , 'Ｆ' , 'f' , 'ｆ' ],
		[  'G' , 'Ｇ' , 'g' , 'ｇ' ],
		[  'H' , 'Ｈ' , 'h' , 'ｈ' ],
		[  'I' , 'Ｉ' , 'i' , 'ｉ' ],
		[  'J' , 'Ｊ' , 'j' , 'ｊ' ],
		[  'K' , 'Ｋ' , 'k' , 'ｋ' ],
		[  'L' , 'Ｌ' , 'l' , 'ｌ' ],
		[  'M' , 'Ｍ' , 'm' , 'ｍ' ],
		[  'N' , 'Ｎ' , 'n' , 'ｎ' ],
		[  'O' , 'Ｏ' , 'o' , 'ｏ' ],
		[  'P' , 'Ｐ' , 'p' , 'ｐ' ],
		[  'Q' , 'Ｑ' , 'q' , 'ｑ' ],
		[  'R' , 'Ｒ' , 'r' , 'ｒ' ],
		[  'S' , 'Ｓ' , 's' , 'ｓ' ],
		[  'T' , 'Ｔ' , 't' , 'ｔ' ],
		[  'U' , 'Ｕ' , 'u' , 'ｕ' ],
		[  'V' , 'Ｖ' , 'v' , 'ｖ' ],
		[  'W' , 'Ｗ' , 'w' , 'ｗ' ],
		[  'X' , 'Ｘ' , 'x' , 'ｘ' ],
		[  'Y' , 'Ｙ' , 'y' , 'ｙ' ],
		[  'Z' , 'Ｚ' , 'z' , 'ｚ' ],
	);

	# テーブル作成
	foreach $list (@table){
		foreach $char (@$list){
			$ilist{$char} = $list->[0];
		}
	}

	# return
	return(\%ilist);
} # itable end

#---
exit;
