#!/usr/local/bin/perl

# strict
use strict;
use vars qw(%init);

# SQLite
use DBI;
# PetitTemplate
use PetitTemplate;

# flatseek-search.cgi (2009/08/16)

# 初期設定
%init = (

	# スクリプトのURL
	self => 'http://www.xxxxxxxx.xx.xx/~xxxxxxxx/cgi-bin/flatseek/flatseek-search.cgi',

	# 検索データベースのパス
	db   => './flatseek-db.sqlite',

	# 検索ディレクトリのURL (最後に / を入れない)
	url  => 'http://www.xxxxxxxx.xx.xx/~xxxxxxxx',

	# gzipのパス (gzip圧縮転送機能使用時のみ)
	gzip => '',

	# 表示テンプレート
	tmpl => {
		main  => './tmpl/main.tmpl.html',
		navi  => './tmpl/navi.tmpl.html',
		frame => './tmpl/frame.tmpl.html',
	},

	# テンプレート変数
	vars => {
		title => 'flatseek β0.02ver.',
	},

); # init

#---

# main

# バージョン表記
$init{'ver'} = "<a href=\"http://hp.vector.co.jp/authors/VA040208/\">flatseek β0.02ver.</a>";

{

	# 入力情報,検索設定,データベース
	my(%form,%query,$dbh);
	# ファイル一覧,ファイルデータ,階層データ,年月データ,検索結果一覧
	my(@flist,%fitem,%plist,$ptree,%dlist,$dtree,$frslt);

	# 検索設定取得
	&input(\%form);
	&loadq(\%query,\%form,&pinfo());

	# フレーム表示
	&frame(\%query) if($form{'f'}); # exit
	# 検索URL移動
	&redir(\%query) if($form{'r'}); # exit

	# データベース取得
	$dbh = &db_open();

	# ファイル情報取得
	&loadg($dbh,\@flist,\%fitem,\%plist,\%dlist);
	# ファイル情報補正
	($ptree,$dtree) = &gtrim(\%plist,\%dlist);

	# 検索ナビhtml表示
	&pnavi($dbh,\%query,$ptree,$dtree) if($form{'n'}); # db_exit

	# ファイル全文検索
	($frslt) = &fgrep($dbh,\%query,\@flist,\%fitem);

	# 検索結果html表示
	&prslt($dbh,\%query,$ptree,$dtree,$frslt); # db_exit

	# データベース終了
	&db_exit($dbh);

}

exit;

#---

# subroutine

# ファイル情報取得
sub loadg{
	# データベース,ファイルデータ,階層データ,年月データ
	my($dbh,$flist,$fitem,$plist,$dlist) = @_;
	# 取得ルーチン,取得データ,個別ファイル
	my($sql,$data,$item);
	# 階層テンポラリ
	my($path,$name,$date);

	# データ取得処理
	$sql = &db_select($dbh,['id','dir','name','url','type','pid','date','size','title','tag','tid','desc']);
	while($data = $sql->()){
		$item = $fitem->{$data->{'id'}} = &fitem($data);
		$date = $item->('date');
		$path = $plist;
		push(@$flist,$item);
		push(@{$dlist->{ $date->(['%Y']) }->{ $date->(['%m']) }},$item);
		foreach $name (@{ $item->('path') }){ $path = ($path->{$name} ||= {}); }
		push(@{$path->{'.filelist'}},$item);
	}
} # loadg end

# ファイル情報補正
sub gtrim{
	# 階層データ,年月データ
	my($plist,$dlist) = @_;
	# 階層ツリー,年月ツリー
	my($ptree,$dtree);

	# 階層情報補正
	$ptree = &ptrim($plist);
	# 年月情報補正
	$dtree = &dtrim($dlist);

	# return
	return($ptree,$dtree);
} # gtrim end

# ファイル検索処理
sub fgrep{
	# データベース,検索設定,ファイルデータ
	my($dbh,$query,$flist,$fitem) = @_;
	# 取得ルーチン,検索実行処理,検索正規表現,検索結果一覧
	my($sql,$qexec,$qregex,@frslt);
	# 検索合致確認,ファイル情報
	my(%chk,$rslt,$data,$item,$id,$text);
	# 検索設定テンポラリ
	my($c,$y,$m) = @{$query}{'c','y','m'};

	# 検索キー解析
	($qexec,$qregex) = &qparse($query->{'q'});

	# 概要検索処理
	foreach $item (@$flist){
		next if($c && (index($item->('dir'),$c) != 0));
		next if(($y || $m) && (!$item->('date')->('search')->([$y,$m])));
		push(@frslt,$item);
	}

	# 本文検索処理
	if($qexec){
		$sql = &db_select($dbh,['id','text']);
		%chk = map { $_->('id') => 1 } @frslt;
		@frslt = ();
		while($data = $sql->()){
			$id   = $data->{'id'};
			next if(!$chk{$id});
			$item = $fitem->{$id};
			$text = $data->{'text'};
			$item->('data')->([ text => $text ]);
			$rslt = $qexec->($item);
			$item->('data')->([]);
			next if(!$rslt);
			$text = &gtext($text,$qregex);
			$item->('data')->([ text => $text ]);
			push(@frslt,$item);
		}
	}

	# return
	return(\@frslt);
} # fgrep end

# 検索結果html表示
sub prslt{
	# データベース,検索設定,階層データ,年月データ,検索結果一覧
	my($dbh,$query,$ptree,$dtree,$frslt) = @_;
	# 表示テンポラリ
	my(%rel);

	# データベース終了
	&db_exit($dbh);

	# html書き出し
	$rel{'path'} = &rpath($query,$frslt);
	$rel{'date'} = &rdate($query,$frslt);
	&p_tmpl('main',{
		query => $query,
		qform => {
			c => sub { $ptree->('form',[ $query ]); },
			d => sub { $dtree->('form',[ $query ]); },
		},
		qrslt => {
			i => sub { &vlist('tid' ,$query,$frslt) },
			t => sub { &vlist('type',$query,$frslt) },
			c => sub { &vtree($ptree,'srch1',$query,$frslt) },
			d => sub { &vtree($dtree,'srch1',$query,$frslt) },
			y => sub { &vtree($ptree,'srch2',$query,$frslt) },
			z => sub { &vtree($dtree,'srch2',$query,$frslt) },
		},
		qdata => {
			cnt => scalar(@$frslt),
			rel => \%rel,
		},
	});
} # prslt end

# 検索ナビhtml表示
sub pnavi{
	# データベース,検索設定,階層データ,年月データ
	my($dbh,$query,$ptree,$dtree) = @_;

	# データベース終了
	&db_exit($dbh);

	# html書き出し
	&p_tmpl('navi',{
		query => $query,
		qnavi => {
			c => sub { $ptree->('navi',[ $query ]); },
			d => sub { $dtree->('navi',[ $query ]); },
		},
	});
} # pnavi end

#---

# 検索結果ツリー表示
sub vtree{
	# オブジェクト,検索形式,検索設定,検索結果一覧
	my($etree,$name,$query,$frslt) = @_;
	# テンポラリ変数
	my($item,$html,$fcnt);

	# 検索合致設定
	foreach $item (@$frslt){ $item->('data')->([ search_hit => 1 ]); }

	# 表示形式分岐
	$html = $etree->($name,[ $query ]);

	# 合致設定削除
	foreach $item (@$frslt){ $item->('data')->([ search_hit => 0 ]); }

	# return
	return($html);
} # vtree end

# 検索結果リスト表示
sub vlist{
	# 分類形式(tid,type),検索設定,検索結果一覧
	my($type,$query,$frslt) = @_;
	# ファイル情報,htmlデータ,分類一覧,分類題名
	my($item,$html,%clist,$name);

	# ファイル分類
	foreach $item (@$frslt){
		foreach $name ($item->($type)){
			push(@{$clist{$name}},$item);
		}
	}

	# 分類html作成
	$html  = "<ul>\n";
	foreach $name (sort(keys(%clist))){
		$html .= &hhtml("#$name",$name,$name,1,scalar(@{$clist{$name}}));
		$html .= "<ul>\n" . join('',map { $_->('list',[ $query ]) } @{ &fsort(2,$clist{$name}) }) . "</ul>\n";
	}
	$html .= "</ul>\n";

	# return
	return($html);
} # vlist end

#---

# 階層絞り込み作成
sub rpath{
	# 検索設定,検索結果一覧
	my($query,$frslt) = @_;
	# 階層分類,絞り込み配列
	my(%pchk,@ptmp,@rslt);
	# テンポラリ変数
	my($item,$path,$name,$chk,$cnt,$url);

	# 階層分類作成
	foreach $item (@$frslt){
		$path = \%pchk;
		foreach $name (@{ $item->('path') }){ $path = ($path->{$name} ||= {}); }
		$path->{'.filelist'}++;
	}
	$path = \%pchk;
	while(1){
		$chk = delete($path->{'.filelist'});
		$cnt = scalar(keys(%$path));
		last if($cnt == 0);
		if((!$chk) && ($cnt == 1)){
			($name,$path) = %$path;
			push(@ptmp,$name);
		}else{
			foreach $name (sort(keys(%$path))){
				$cnt = &rpchk($path->{$name});
				$url = &squrl(0,$query,{ c => join('/','',@ptmp,$name) });
				push(@rslt,{ url => $url, name => $name, cnt => $cnt });
			}
			return({ name => (join('/',@ptmp) || '/'), list => \@rslt });
		}
	}
	return();
} # rpath end

# 階層分類巻き上げ
sub rpchk{
	my($path) = @_;
	my($n,$i);
	my $cnt = delete($path->{'.filelist'}) || 0;
	while(($n,$i) = each(%$path)){ $cnt += &rpchk($i); }
	return($cnt);
} # rpchk end

# 年月絞り込み作成
sub rdate{
	# 検索設定,検索結果一覧
	my($query,$frslt) = @_;
	# 年月分類,絞り込み配列
	my(%ychk,%mchk,@rslt);
	# テンポラリ変数
	my($item,$date,$y,$m,$url);

	# 年月分類作成
	foreach $item (@$frslt){
		$date = $item->('date');
		$ychk{ $date->(['%Y']) }++;
		$mchk{ $date->(['%m']) }++;
	}
	if(1 < scalar(keys(%ychk))){
		foreach $y (sort(keys(%ychk))){
			$url = &squrl(0,$query,{ y => $y, m => '' });
			push(@rslt,{ url => $url, name => $y, cnt => $ychk{$y} });
		}
		return({ type => 1, list => \@rslt, year => '' });
	}elsif(1 < scalar(keys(%mchk))){
		($y) = keys(%ychk);
		foreach $m (sort(keys(%mchk))){
			$url = &squrl(0,$query,{ y => $y, m => $m });
			push(@rslt,{ url => $url, name => $m, cnt => $mchk{$m} });
		}
		return({ type => 2, list => \@rslt, year => $y });
	}
	return();
} # rdate end

#---

# 個別ファイル情報
sub fitem{
	# ファイル情報
	my($item) = @_;
	my(%data);
	# 各種実行処理
	$item->{'date'} = &mtype($item->{'date'},\&idate,\&edate);
	$item->{'path'} = &ipath($item->{'dir'});
	$item->{'data'} = &idata(\%data);
	my %esub = (
		list => sub { &fhtml($item,\%data) },
		srch => sub { ($data{'search_hit'}) ? &fhtml($item,\%data) : ''; },
	);

	# オブジェクト作成
	return(sub {
		my($name,$args) = &qitem(2,@_);
		return($esub{$name}->($args)) if($esub{$name});
		return($item->{$name})        if(exists($item->{$name}));
		return($data{$name}  )        if(exists($data{$name}));
		return(undef);
	});
} # fitem end

# ファイル階層情報
sub pitem{
	# ディレクトリ名,階層段階配列,階層データ
	my($name,$path,$plist) = @_;
	# 個別階層データ,階層ルーチン,ファイル一覧,直下階層一覧
	my(%pobj,%esub,$file,@child);
	# ファイル数,個別データ
	my($cnt1,$cnt2,$dir,$item);

	# 階層情報確認
	$file = &fsort(2,($plist->{'.filelist'} || []));
	$cnt1 = scalar(@$file);
	push(@$path,$name);
	foreach $dir (sort(grep { /^[^\.]/ } keys(%$plist))){
		($item,$cnt2) = &pitem($dir,[@$path],$plist->{$dir});
		push(@child,$item);
		$cnt1 += $cnt2;
	}

	# オブジェクト作成
	%pobj = ( name => $name, path => $path, file => $file, child => \@child, count => $cnt1 );
	%esub = &phtml(\%pobj);
	return(sub {
		my($name,$args) = &qitem(2,@_);
		return($esub{$name}->($args)) if($esub{$name});
		return($pobj{$name})          if(exists($pobj{$name}));
		return(undef);
	},$cnt1);
} # pitem end

# 階層構造html作成
sub phtml{
	# 階層データ
	my($pobj) = @_;
	# テンポラリ変数
	my($html,$temp,$cnt1,$cnt2);
	my $name = $pobj->{'name'} || '/';
	my $path = join('/',@{$pobj->{'path'}});
	my $view = sub {
		my($q) = @_;
		return(1) if($q->{'c'} eq '');
		return(1) if(index($path,$q->{'c'}) == 0);
		return(0);
	};
	my $pchk = sub {
		my($q) = @_;
		return(1) if($view->($q) && ($path ne $q->{'c'}));
		return(1) if(($q->{'c'} ne '') && (index($q->{'c'},$path) < 0));
		return(0);
	};
	# 階層共通ループ
	my $lsub = sub {
		my($name,$args) = @_;
		my(@html,$item,$temp,$cnt1,$cnt2);
		$cnt1 = 0;
		foreach $item (@{$pobj->{'child'}}){
			($temp,$cnt2) = $item->($name,$args);
			push(@html,$temp);
			$cnt1 += $cnt2;
		}
		return(\@html,$cnt1);
	};
	# 記事共通ループ
	my $fsub = sub {
		my($name,$args) = @_;
		my(@html,$item,$temp);
		foreach $item (@{$pobj->{'file'}}){
			$temp = $item->($name,$args);
			if($temp){ push(@html,$temp); }
		}
		return(\@html,scalar(@html));
	};

	# htmlルーチン作成
	return(
		form  => sub {
			my($args) = @_;
			my($query) = @$args;
			($temp,$cnt1) = $lsub->('form',$args);
			$cnt1 += scalar(@{$pobj->{'file'}});
			$html  = &ohtml($path,($query->{'c'} eq $path),$#{$pobj->{'path'}},$name,$cnt1);
			return(($html . join('',@$temp)),$cnt1);
		},
		navi  => sub {
			my($args) = @_;
			my($query) = @$args;
			($temp,$cnt1) = $lsub->('navi',$args);
			$cnt1 += scalar(@{$pobj->{'file'}});
			$html  = &nhtml(&squrl(0,$query,{ c => $path }),$name,$cnt1);
			return(($html . "<ul>\n" . join('',@$temp) . "</ul>\n"),$cnt1);
		},
		srch1 => sub {
			my($args) = @_;
			my($query) = @$args;
			($temp,$cnt1) = $lsub->('srch1',$args);
			$html  = join('',@$temp);
			($temp,$cnt2) = $fsub->('srch' ,$args);
			$html .= join('',@$temp);
			$cnt1 += $cnt2;
			$html  = &hhtml(&squrl(0,$query,{ c => $path }),$path,$name,$view->($query),$cnt1)
			       . (($pchk->($query) && ($cnt1 == 0)) ? '' : "<ul>\n$html</ul>\n");
			return($html,$cnt1);
		},
		srch2 => sub {
			my($args) = @_;
			my($query) = @$args;
			($temp,$cnt1) = $lsub->('srch2',$args);
			$html  = join('',@$temp);
			($temp,$cnt2) = $fsub->('srch' ,$args);
			return($html,$cnt1) if($cnt2 == 0);
			$html  = &hhtml(&squrl(0,$query,{ c => $path }),$path,($path || '/'),1,$cnt2)
			       . "<ul>\n" . join('',@$temp) . "</ul>\n" . $html;
			return($html,($cnt1 + $cnt2));
		},
	);
} # phtml end

# ファイル階層補正
sub ptrim{
	# 階層データ
	my($plist) = @_;
	# トップオブジェクト,階層ルーチン,テンポラリ変数
	my($tsub,%esub,$html);

	# 階層全体補正
	($tsub,undef) = &pitem('',[],$plist);

	# 各種実行処理
	%esub = (
		form  => sub {
			my($args) = @_;
			my($query) = @$args;
			($html,undef) = $tsub->('form' ,$args);
			return($html);
		},
		navi  => sub {
			my($args) = @_;
			my($query) = @$args;
			($html,undef) = $tsub->('navi' ,$args);
			return("<ul>\n$html</ul>\n");
		},
		srch1 => sub {
			my($args) = @_;
			my($query) = @$args;
			($html,undef) = $tsub->('srch1',$args);
			return("<ul>\n$html</ul>\n");
		},
		srch2 => sub {
			my($args) = @_;
			my($query) = @$args;
			($html,undef) = $tsub->('srch2',$args);
			return("<ul>\n$html</ul>\n");
		},
	);

	# オブジェクト作成
	return(sub {
		my($name,$args) = &qitem(2,@_);
		return($esub{$name}->($args)) if($esub{$name});
		return(undef);
	});
} # ptrim end

# 年月階層情報補正
sub dtrim{
	# 年月データ
	my($dlist) = @_;
	# 年一覧,年別月一覧,ファイル一覧,年別記事数
	my(@year,@month,$file,$cnt);
	# 階層ルーチン,テンポラリ変数
	my(%esub,$y,$m,$item,$html,$fcnt);
	# 年月共通ループ
	my $lsub = sub {
		my($name,$args) = @_;
		my(@html,$item,$temp,$cnt1,$cnt2);
		$cnt1 = 0;
		foreach $item (@year){
			($temp,$cnt2) = $item->($name,$args);
			push(@html,$temp);
			$cnt1 += $cnt2;
		}
		return(\@html,$cnt1);
	};

	# 年月情報確認
	foreach $y (reverse(sort(keys(%$dlist)))){
		$cnt = 0;
		@month = ();
		foreach $m (reverse(sort(keys(%{$dlist->{$y}})))){
			$file = &fsort(1,$dlist->{$y}->{$m});
			$item = { year => $y, month => $m, file => $file, count => scalar(@$file) };
			$cnt += scalar(@$file);
			push(@month,&mitem($item));
		}
		$item = { year => $y, month => [@month], count => $cnt };
		push(@year,&yitem($item));
	}

	# 各種実行処理
	%esub = (
		form  => sub {
			my($args) = @_;
			my($query) = @$args;
			($html,$fcnt) = $lsub->('form' ,$args);
			return(&ohtml('','',0,'/',$fcnt) . join('',@$html));
		},
		navi  => sub {
			my($args) = @_;
			my($query) = @$args;
			($html,$fcnt) = $lsub->('navi' ,$args);
			return("<ul>\n" . &nhtml(&squrl(0,$query,{ y => '', m => '' }),'/',$fcnt)
			      ."<ul>\n" . join('',@$html) . "</ul>\n</ul>\n");
		},
		srch1 => sub {
			my($args) = @_;
			my($query) = @$args;
			($html,$fcnt) = $lsub->('srch1',$args);
			return("<ul>\n" . &hhtml(&squrl(0,$query,{ y => '', m => '' }),'/','/',(($query->{'y'} eq '') && ($query->{'m'} eq '')),$fcnt)
		          ."<ul>\n" . join('',@$html) . "</ul>\n</ul>\n");
		},
		srch2 => sub {
			my($args) = @_;
			my($query) = @$args;
			($html,undef) = $lsub->('srch2',$args);
			return("<ul>\n" . join('',@$html) . "</ul>\n");
		},
	);

	# オブジェクト作成
	return(sub {
		my($name,$args) = &qitem(2,@_);
		return($esub{$name}->($args)) if($esub{$name});
		return(undef);
	});
} # dtrim end

# 年階層情報作成
sub yitem{
	# 年階層データ
	my($yobj) = @_;
	# 階層ルーチン,テンポラリ変数
	my(%esub,$html,$temp,$fcnt);
	my $year = $yobj->{'year'};
	my $view = sub {
		my($q) = @_;
		return(1) if( $q->{'y'} eq '');
		return(1) if(($q->{'y'} eq $year) && ($q->{'m'} eq ''));
		return(0);
	};
	my $mchk = sub {
		my($q) = @_;
		my($Y,$M) = @{$q}{'y','m'};
		return(1) if(($Y ne '') && ($Y ne $year));
		return(1) if(($Y eq '') && ($M eq ''));
		return(0);
	};
	# 月共通ループ
	my $lsub = sub {
		my($name,$args) = @_;
		my(@html,$item,$temp,$cnt1,$cnt2);
		$cnt1 = 0;
		foreach $item (@{$yobj->{'month'}}){
			($temp,$cnt2) = $item->($name,$args);
			push(@html,$temp);
			$cnt1 += $cnt2;
		}
		return(\@html,$cnt1);
	};

	# 各種実行処理
	%esub = (
		form  => sub {
			my($args) = @_;
			my($query) = @$args;
			($temp,$fcnt) = $lsub->('form',$args);
			$html = &ohtml($year,(($query->{'y'} eq $year) && ($query->{'m'} eq '')),1,$year,$fcnt);
			return(($html . join('',@$temp)),$fcnt);
		},
		navi  => sub {
			my($args) = @_;
			my($query) = @$args;
			($temp,$fcnt) = $lsub->('navi',$args);
			$html = &nhtml(&squrl(0,$query,{ y => $year, m => '' }),$year,$fcnt);
			return(($html . "<ul>\n" . join('',@$temp) . "</ul>\n"),$fcnt);
		},
		srch1 => sub {
			my($args) = @_;
			my($query) = @$args;
			($temp,$fcnt) = $lsub->('srch1',$args);
			$html = join('',@$temp);
			$html = &hhtml(&squrl(0,$query,{ y => $year, m => '' }),$year,$year,$view->($query),$fcnt)
			      . (($mchk->($query) && ($fcnt == 0)) ? '' : "<ul>\n$html</ul>\n");
			return($html,$fcnt);
		},
		srch2 => sub {
			my($args) = @_;
			my($query) = @$args;
			($temp,$fcnt) = $lsub->('srch2',$args);
			return(join('',@$temp),$fcnt);
		},
	);

	# オブジェクト作成
	return(sub {
		my($name,$args) = &qitem(2,@_);
		return($esub{$name}->($args)) if($esub{$name});
		return($yobj->{$name})        if(exists($yobj->{$name}));
		return(undef);
	});
} # yitem end

# 月階層情報作成
sub mitem{
	# 月階層データ
	my($mobj) = @_;
	# 階層ルーチン,テンポラリ変数
	my(%esub,$html,$temp,$fcnt);
	my $year = $mobj->{'year'};
	my $month= $mobj->{'month'};
	my $path = sprintf('%04d/%02d',$year,$month);
	my $view = sub {
		my($q) = @_;
		my($Y,$M) = @{$q}{'y','m'};
		my($y,$m) = ($year,$month);
		return(1) if(($Y eq '') && ($M eq ''));
		return(1) if(($Y eq $y) && ($M eq ''));
		return(1) if(($Y eq '') && ($M eq $m));
		return(1) if(($Y eq $y) && ($M eq $m));
		return(0);
	};
	# 記事共通ループ
	my $lsub = sub {
		my($name,$args) = @_;
		my(@html,$item,$temp);
		foreach $item (@{$mobj->{'file'}}){
			$temp = $item->($name,$args);
			if($temp){ push(@html,$temp); }
		}
		return(\@html,scalar(@html));
	};

	# 各種実行処理
	%esub = (
		form  => sub {
			my($args) = @_;
			my($query) = @$args;
			$fcnt = scalar(@{$mobj->{'file'}});
			$html = &ohtml($path,(($query->{'y'} eq $year) && ($query->{'m'} eq $month)),2,$month,$fcnt);
			return($html,$fcnt);
		},
		navi  => sub {
			my($args) = @_;
			my($query) = @$args;
			$fcnt = scalar(@{$mobj->{'file'}});
			$html = &nhtml(&squrl(0,$query,{ y => $year, m => $month }),$month,$fcnt);
			return($html,$fcnt);
		},
		srch1 => sub {
			my($args) = @_;
			my($query) = @$args;
			($temp,$fcnt) = $lsub->('srch',$args);
			$html = join('',@$temp);
			$html = &hhtml(&squrl(0,$query,{ y => $year, m => $month }),$path,$month,$view->($query),$fcnt)
			      . (($fcnt == 0) ? '' : "<ul>\n$html</ul>\n");
			return($html,$fcnt);
		},
		srch2 => sub {
			my($args) = @_;
			my($query) = @$args;
			($temp,$fcnt) = $lsub->('srch',$args);
			return('',0) if($fcnt == 0);
			$html = &hhtml(&squrl(0,$query,{ y => $year, m => $month }),$path,$path,1,$fcnt)
			      . "<ul>\n" . join('',@$temp) . "</ul>\n";
			return($html,$fcnt);
		},
	);

	# オブジェクト作成
	return(sub {
		my($name,$args) = &qitem(2,@_);
		return($esub{$name}->($args)) if($esub{$name});
		return($mobj->{$name})        if(exists($mobj->{$name}));
		return(undef);
	});
} # mitem end

#---

# 日付情報解析
sub idate{
	my($date) = @_;
	my(%list);

	# 個別日付解析
	@list{'Y','m','d','H','M','S'} = unpack('A4A2A2A2A2A2',$date);
	return(sub {
		my($args) = &qitem(1,@_);
		my $text  = $args->[0] || '';
		return($date) if($text eq '');
		$text =~ s/%([YmdyHMS])/$list{$1}/g;
		return($text);
	});
} # idate end

# 日付各種処理
sub edate{
	my($mode,$list) = @_;

	# 日付統合処理
	if($mode eq 'search'){
		return(sub {
			my($args) = &qitem(1,@_);
			my($y,$m) = @$args;
			my($i);
			foreach $i (@$list){
				next if($y && ($i->(['%Y']) ne $y));
				next if($m && ($i->(['%m']) ne $m));
				return(1);
			}
			return(0);
		});
	}
	return(undef);
} # edate end

# 階層情報解析
sub ipath{
	my($dir) = @_;
	my @path = split(/\/+/,$dir); shift(@path); return(\@path);
} # ipath end

# ファイルソート
sub fsort{
	# 動作モード(0=ファイル名,1=更新時刻,2=題名),ソート配列
	my($mode,$list) = @_;
	my %isub = ( 0 => sub { $_[0]->('name') }, 1 => sub { $_[0]->('date')->(['']) }, 2 => sub { $_[0]->('title') } );
	my %temp = map  { $_->('id') => $isub{$mode}->($_) } @$list;
	my @rslt = sort { $temp{ $a->('id') } cmp $temp{ $b->('id') } } @$list;
	return(\@rslt);
} # fsort end

# 検索合致行取得
sub gtext{
	# ファイル本文,検索正規表現
	my($text,$qregex) = @_;

	# 全文検索処理
	return('') if(!$qregex);
	return(join("<br>\n",grep { s/$qregex/$1<span class=\"match\">$2<\/span>/g } split(/\n+/,$text))); #"
} # gtext end

#---

# ファイルhtml作成
sub fhtml{
	# ファイル情報
	my($item,$data) = @_;

	# html作成
	return( "<li><a href=\"$init{'url'}$item->{'url'}\" title=\"$item->{'desc'}\">$item->{'title'}</a> "
	      . "<small>" . $item->{'date'}->(['%Y/%m/%d %H:%M']) . " $item->{'size'}byte</small>"
	      . (($data->{'text'} ne '') ? "<br>\n<blockquote> $data->{'text'} </blockquote>" : "") . "</li>\n");
} # fhtml end

# 検索分類html作成
sub hhtml{
	my($href,$link,$name,$view,$cnt) = @_;
	return(sprintf("<li><a href=\"%s\" name=\"%s\"><b>%s</b></a>%s</li>\n",$href,$link,$name,($view ? " ($cnt)" : '')));
} # hhtml end

# フォームhtml作成
sub ohtml{
	my($path,$slct,$ttab,$name,$cnt) = @_;
	$ttab = '　' x $ttab;
	return(sprintf("<option value=\"%s\" %s>%s%s (%d)</option>\n",$path,($slct ? 'selected' : ''),$ttab,$name,$cnt));
} # ohtml end

# 検索ナビhtml作成
sub nhtml{
	my($href,$name,$cnt) = @_;
	return(sprintf("<li><a href=\"%s\" target=\"search\"><b>%s</b></a> (%d)</li>\n",$href,$name,$cnt));
} # nhtml end

#---

# データ属性設定
sub idata{
	# 拡張データ
	my($data) = @_;

	# 更新ルーチン作成
	return(sub {
		my($args) = &qitem(1,@_);
		%$data = (scalar(@$args) == 0) ? () : ( %$data, @$args ) if((scalar(@$args) % 2) == 0);
	});
} # idata end

# 複数データ解析
sub mtype{
	# 属性キー,処理ルーチン
	my($text,$exec,$util) = @_;
	# 属性配列,作成ルーチン
	my(@list,$csub,$usub);

	# オブジェクト作成
	@list = split(/\,/,$text);
	$csub = $exec->($list[0]);
	$usub = sub { return([ (map { $exec->($_) } @list) ]); };
	return(sub {
		my($name,$args) = &qitem(2,@_);
		return($exec->($list[$name - 1])) if($name =~ /^\d+$/);
		return($util->($name,$usub->() )) if($name ne '');
		return(scalar( @list))            if($args->[0] eq 'len');
		return($csub->($args));
	});
} # mtype end

# データ引数解析
sub qitem{
	my($mode,@temp) = @_;
	my($name,$args);
	my %back = ( 0 => sub { ($name) }, 1 => sub { ($args) }, 2 => sub { ($name,$args) } );
	$name = (ref($temp[0]) eq '')      ? shift(@temp) : '';
	$args = (ref($temp[0]) eq 'ARRAY') ? shift(@temp) : [];
	return((ref($back{$mode}) eq 'CODE') ? $back{$mode}->() : ());
} # qitem end

#---

# 検索設定取得
sub loadq{
	# 検索設定,フォーム,PATH_INFO
	my($query,$form,$path) = @_;

	# フォーム解析
	$query->{'c'} =  $form->{'c'} || '';
	$query->{'q'} =  $form->{'q'} || '';
	$query->{'y'} = ($form->{'y'} =~ /^(\d{4})$/) ? $1 : '';
	$query->{'m'} = ($form->{'m'} =~ /^(\d{2})$/) ? $1 : '';
	$query->{'v'} = ($form->{'v'} =~ /^([a-z])$/) ? $1 : 'i';

	# PATH_INFO解析
	if($path ne ''){
		$query->{'v'} = ($path =~ s/\/([a-z])$//) ? $1 : 'i';
		$query->{'m'} = ($path =~ s/\/(\d{2})$//) ? $1 : '';
		$query->{'y'} = ($path =~ s/\/(\d{4})$//) ? $1 : '';
		$query->{'c'} = join('/',map { &urldec($_) } split(/\/+\.*/,$path));
	}

	# 追加設定解析
	local $_;
	@{$query}{'y','m'} = ($form->{'ym'} =~ /^(\d{4})\/(\d{2})$/) ? ($1,$2)
	: ($form->{'ym'} =~ /^(\d{4})$/) ? ($1,'') : ('','') if($form->{'ym'});
	@{$query}{'c','y','m','q'} = ('','','','') if($form->{'rs'});
	foreach('i','t','c','d','y','z'){ $query->{'v'} = $_ if($form->{"v$_"}); }
} # loadq end

# 検索URL作成
sub squrl{
	# 動作モード,検索設定,書き換え設定
	my($mode,$query,$qwrap) = @_;
	# URL配列
	my($join,@path,@qstr);

	# URL作成
	%$qwrap = ( %$query,%$qwrap );
	$join = ($mode) ? '&' : '&amp;';
	@qstr = ([ 'q',$qwrap->{'q'} ]);
	@path = map { ($_) ? &urlenc($_) : () } (split(/\/+\.*/,$qwrap->{'c'}),$qwrap->{'y'},$qwrap->{'m'},$qwrap->{'v'});
	return(join('/',$init{'self'},@path) . '?' . join($join,map { &urlenc($_->[0]) . '=' . &urlenc($_->[1]) } @qstr));
} # squrl end

#---

# 検索キー解析
sub qparse{
	# 検索キー
	my($query) = @_;
	# 検索配列,検索正規表現
	my(@qlist,$qregex,@qrtemp);
	# テンポラリ変数
	my(@temp,$item,@list);
	my $tree = \@qlist;

	# 検索キー分割
	$query = &escape(1,$query);
	$query =~ s/(?:\s|\　)+/ /g;
	while($query =~ /([\(\)]|([\"\'])(?:[^\2]*?)\2|[^\s\"\'\(\)]+(?:[\"\']+[^\s\"\'\(\)]+)*[\"\']*)/g){ push(@temp,$1); } #"
	while(defined($item = shift(@temp))){
		if(    $item eq '('){ push(@list,$tree); push(@$tree,[]); $tree = $tree->[$#$tree];
		}elsif($item eq ')'){ $tree = pop(@list) if(0 < scalar(@list));
		}elsif((lc($item) eq 'and') || ($item eq '&&')){ push(@$tree,{ type => 'and' });
		}elsif((lc($item) eq 'or' ) || ($item eq '||')){ push(@$tree,{ type => 'or'  });
		}elsif($item =~ /^\"([^\"]*?)\"$/){ &qpitem(1,&escape(0,$1),$tree,\@qrtemp); #"
		}elsif($item =~ /^\'([^\']*?)\'$/){ &qpitem(0,&escape(0,$1),$tree,\@qrtemp); #'
		}elsif($item =~ /\S/){ &qpitem(1,&escape(0,$item),$tree,\@qrtemp); }
	}
	return() if(scalar(@qlist) == 0);
	if(0 < scalar(@qrtemp)){ $qregex = join('|',@qrtemp); $qregex = qr/((?:\G|>)[^<]*?)($qregex)/i; }
	return(sub { &sqexec(\@qlist,$_[0]) },$qregex);
} # qparse end

# 個別キー解析
sub qpitem{
	# 動作モード,検索キー,検索配列,正規表現配列
	my($mode,$query,$qlist,$qrtemp) = @_;
	# 検索設定
	my($type,$name) = (0,'');

	# 個別キー解析
	if($query =~ /^(-\:)?(\w+)\:(.*?)$/){ ($type,$name,$query) = (($1 ? 1 : 0),$2,$3); }
	if(&oquery($name,$query)){
		push(@$qlist,{ type => 'option', mode => $type, name => $name, query => $query });
	}elsif(($query) = &quote1($mode,$query)){
		push(@$qlist,{ type =>  'query', mode => $type, name => $name, query => qr/(?:(?:^|>)[^<]*?)$query/i });
		push(@$qrtemp,$query) if(($type == 0) && (($name eq '') || ($name eq 'text')));
	}
} # qpitem end

# 正規表現作成
sub quote1{
	my($mode,$query) = @_;
	my($s,$e,@qtemp,@qlist,$item);
	my %w = ('*' => '(?:[^<>\0\n]*?)', ' ' => '(?:(?:[\t\ ]|\　)*?)', '&quot;' => '(?:&quot;|&#39;)', '&#39;' => '(?:&quot;|&#39;)');
	return(($query =~ /\S/) ? (join('',map { $w{$_} || quotemeta($_) } split(/(\s+|&quot;|&#39;)/,$query))) : ()) if($mode == 0);
	$s = ($query =~ s/^\^//) ? '(?:(?:^|[\0\n])(?:[\t\ ]|\　)*)' : '';
	$e = ($query =~ s/\$$//) ? '(?:(?:[\t\ ]|\　)*(?:[\0\n]|$))' : '';
	return() if($query !~ /[^\s\*]/);
	$query =~ s/\*+/\*/g;
	$query =~ s/^[\s\*]+//;
	$query =~ s/[\s\*]+$//;
	@qtemp = split(/([\s\*]|\[(?:[^\]]*?)\]|\{(?:[^\}]*?)\})/,$query);
	while(defined($item = shift(@qtemp))){
		if($w{$item}){ push(@qlist,$w{$item});
		}elsif($item =~ /^\[([^\]]*?)\]$/){ push(@qlist,&quote2(0,$1));
		}elsif($item =~ /^\{([^\}]*?)\}$/){ push(@qlist,&quote2(1,$1));
		}elsif($item =~ /\S/){ push(@qlist,quotemeta($item)); }
	}
	return($s . join('',@qlist) . $e);
}
sub quote2{
	my($mode,$query) = @_;
	return(($query =~ /[^\s\-]/) ? ('(?:[' . join('-',map { quotemeta($_) } split(/\-+/,$query)) . '])') : ()) if($mode == 0);
	return(($query =~ /[^\s\|]/) ? ('(?:'  . join('|',map { &quote1(1,$_) } split(/\|+/,$query)) .  ')') : ()) if($mode == 1);
	return();
}

# 検索オプション
sub oquery{
	my($name,$query) = @_;
	return($query =~ /^\w+$/) if($name eq 'type');
	return($query =~ /^\d+$/) if($name eq 'size');
	return(0);
} # oquery end

#---

# 検索実行処理
sub sqexec{
	my($qlist,$item) = @_;
	my $qlen = scalar(@$qlist);
	return(&qcheck($qlist->[0],$item)) if($qlen == 1);
	return(&qcheck($qlist->[0],$item) && &qcheck($qlist->[1],$item)) if( $qlen == 2);
	return(&qcheck($qlist->[0],$item) && &qcheck($qlist->[2],$item)) if(($qlen == 3) && &qtype($qlist->[1],'and'));
	return(&qcheck($qlist->[0],$item) || &qcheck($qlist->[2],$item)) if(($qlen == 3) && &qtype($qlist->[1],'or' ));
	return(&sqexec([ (&qcheck($qlist->[0],$item) && &qcheck($qlist->[2],$item)),@{$qlist}[3 .. $#$qlist] ],$item)) if((4 <= $qlen) && &qtype($qlist->[1],'and'));
	return(&sqexec([ (&qcheck($qlist->[0],$item) || &qcheck($qlist->[2],$item)),@{$qlist}[3 .. $#$qlist] ],$item)) if((4 <= $qlen) && &qtype($qlist->[1],'or' ));
	return(&sqexec([ (&qcheck($qlist->[0],$item) && &qcheck($qlist->[1],$item)),@{$qlist}[2 .. $#$qlist] ],$item)) if( 3 <= $qlen);
	return(0);
} # sqexec end

# 個別キー分岐
sub qcheck{
	my($query,$item) = @_;
	return(&sqexec($query,$item)) if(ref($query) eq 'ARRAY');
	return(&scheck($query,$item)) if(&qtype($query,'query'));
	return(&ocheck($query,$item)) if(&qtype($query,'option'));
	return(((ref($query) eq '') && $query) ? 1 : 0);
} # qcheck end

# 正規表現検索
sub scheck{
	my($query,$item) = @_;
	my($type,$name,$regex) = @{$query}{'mode','name','query'};
	my($b1,$b2) = ($type == 1) ? (0,1) : (1,0);
	my @chk = ('dir','name','url','title','text');
	my %set = ( dir => 1, name => 1, url => 1, title => 1, text => 1 );
	local $_;
	return(($item->($name) =~ /$regex/) ? $b1 : $b2) if($set{$name});
	foreach(@chk){ return($b1) if($item->($_) =~ /$regex/); } return($b2);
} # scheck end

# オプション検索
sub ocheck{
	my($query,$item) = @_;
	my($type,$name,$opt) = @{$query}{'mode','name','query'};
	my($b1,$b2) = ($type == 1) ? (0,1) : (1,0);
	return(($item->('type') eq $opt) ? $b1 : $b2) if($name eq 'type');
	return(($opt <= $item->('size')) ? $b1 : $b2) if($name eq 'size');
	return(0);
} # ocheck end

# 検索形式取得
sub qtype{
	my($query,$type) = @_;
	return(0) if(ref($query) ne 'HASH');
	return(0) if($query->{'type'} ne $type);
	return(1);
} # qtype end

#---

# データベース処理
sub db_open{ DBI->connect("DBI:SQLite:$init{'db'}",'','',{ RaiseError => 0, PrintError => 1 }); }
sub db_exit{ $_[0]->disconnect(); }

# データベース取得
sub db_select{
	# データベース,取得対象配列
	my($dbh,$list) = @_;
	# 取得ルーチン,取得テンポラリ
	my($sth,@temp,%data);

	# 取得処理
	$sth = $dbh->prepare(sprintf('SELECT %s FROM flatseek;',join(', ',@$list)));
	$sth->execute();
	return(sub {
		@temp = $sth->fetchrow_array();
		return(undef) if(scalar(@temp) != scalar(@$list));
		@data{@$list} = @temp;
		return({ %data });
	});
}

#---

# 入力取得処理
sub input{
	my($form) = @_;
	my($name,$value);
	my $buff  = $ENV{'QUERY_STRING'} || '';

	local $_;
	foreach(split(/&/,$buff)){
		next if(index($_,'=') < 0);
		($name,$value) = split(/=/);
		$form->{$name} = &urldec($value);
	}
} # input end

# パス取得処理
sub pinfo{
	my $req = $ENV{'REQUEST_URI'} || '';
	my $src = $ENV{'SCRIPT_NAME'} || '';
	$req =~ s/^\Q$src\E//;
	$req =~ s/[\?\#].*$//;
	return($req);
} # pinfo end

#---

# 文字列エスケープ
sub escape{
	# 動作モード(0=通常変換,1=戻す),変換する文字列
	my($mode,$text) = @_;

	# 変換処理
	if($mode == 1){
		$text =~ s/&lt;/</g;
		$text =~ s/&gt;/>/g;
		$text =~ s/&quot;/"/g; #"
		$text =~ s/&#39;/'/g;
		$text =~ s/&amp;/&/g;
	}else{
		$text =~ s/&/&amp;/g;
		$text =~ s/</&lt;/g;
		$text =~ s/>/&gt;/g;
		$text =~ s/"/&quot;/g; #"
		$text =~ s/'/&#39;/g;  #'
		$text =~ tr/\x00-\x1f//d;
		$text =~ s/^\s*//;
		$text =~ s/\s*$//;
	}

	# return
	return($text);
} # escape end

# URL文字列変換
sub urldec{
	my($text) = @_;
	$text =~ tr/+/ /;
	$text =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack('H2',$1)/eg;
	$text = &escape(0,$text);
	return($text);
}
sub urlenc{
	my($text) = @_;
	$text = &escape(1,$text);
	$text =~ s/(?:\　)+/ /g;
	$text =~ s/([^\w\s])/'%' . unpack('H2',$1)/eg;
	$text =~ s/\s+/\+/g;
	return($text);
}

#---

# ページ表示処理
sub p_tmpl{
	# テンプレート名,表示パラメータ
	my($tmpl,$param) = @_;
	# PetitTemplate
	my($tobj);

	# テンプレート処理
	$tobj = new PetitTemplate( filename => $init{'tmpl'}->{$tmpl} );
	$tobj->param({ util => { self => $init{'self'}, vars => $init{'vars'} }, ver => $init{'ver'}, %$param });
	print <<HEAD_END;
Content-Type: text/html; charset=UTF-8
Cache-Control: no-cache, no-store, must-revalidate
Pragma: no-cache
HEAD_END
	if(&gzchk()){
		print "Content-Encoding: gzip\n\n";
		open(STDOUT,"| $init{'gzip'} -1 -c") || exit;
		binmode(STDOUT);
		$| = 1;
	}else{
		print "\n";
	}
	print $tobj->output();

	# exit
	exit;
} # p_tmpl end

# 圧縮転送確認処理
sub gzchk{
	my $ae = $ENV{'HTTP_ACCEPT_ENCODING'} || '';
	return(($init{'gzip'} && (-1 < index($ae,'gzip'))) ? 1 : 0);
} # gzchk end

# フレーム表示処理
sub frame{
	my($query) = @_;
	&p_tmpl('frame',{ query => $query });
	exit;
} # frame end

# 検索URL移動処理
sub redir{
	my($query) = @_;
	print "Location: " . &squrl(1,$query,{}) . "\n\n";
	exit;
} # redir end

#---
exit;
